@@ -28,7 +28,12 @@
 ##' @param u_color color for url
 ##' @param u_family font family for url
 ##' @param u_size text size for url
+##' @param u_angle angle for url
+##' @param white_around_sticker default to FALSE. If set to TRUE, it puts
+##'           white triangles in the corners
+##' @param ... additional parameter to geom_pkgname
 ##' @param filename filename to save sticker
+##' @param asp aspect ratio, only works if subplot is an image file
 ##' @param dpi plot resolution
 ##' @return gg object
 ##' @importFrom ggplot2 ggplot
@@ -41,46 +46,58 @@
 ##' p <- ggplot(aes(x = mpg, y = wt), data = mtcars) + geom_point()
 ##' p <- p + theme_void() + theme_transparent()
 ##' sticker(p, package="hexSticker")
-##' @author guangchuang yu
+##' @author Guangchuang Yu 
 sticker <- function(subplot, s_x=.8, s_y=.75, s_width=.4, s_height=.5,
                     package, p_x=1, p_y=1.4, p_color="#FFFFFF", p_family="Aller_Rg", p_size=8,
                     h_size=1.2, h_fill="#1881C2", h_color="#87B13F",
                     spotlight=FALSE, l_x=1, l_y=.5, l_width=3, l_height=3, l_alpha=0.4,
-                    url = "",  u_x=1, u_y=0.08, u_color="black", u_family="Aller_Rg", u_size=1.5,
-                    filename = paste0(package, ".png"), dpi = 300) {
+                    url = "",  u_x=1, u_y=0.08, u_color="black", u_family="Aller_Rg", u_size=1.5, u_angle=30,
+                    white_around_sticker = FALSE, ...,
+                    filename = paste0(package, ".png"), asp=1, dpi = 300) {
+
+    hex <- ggplot() +
+      geom_hexagon(size = h_size, fill = h_fill, color = NA)
 
-    hex <- hexagon(size=h_size, fill=h_fill, color=h_color)
     if (inherits(subplot, "character")) {
         d <- data.frame(x=s_x, y=s_y, image=subplot)
-        sticker <- hex + geom_image(aes_(x=~x, y=~y, image=~image), d, size=s_width)
+        sticker <- hex + geom_image(aes_(x=~x, y=~y, image=~image), d, size=s_width, asp=asp)
     } else {
         sticker <- hex + geom_subview(subview=subplot, x=s_x, y=s_y, width=s_width, height=s_height)
     }
 
+    sticker <- sticker +
+      geom_hexagon(size = h_size, fill = NA, color = h_color)
+
     if(spotlight)
         sticker <- sticker + geom_subview(subview=spotlight(l_alpha), x=l_x, y=l_y, width=l_width, height=l_height)
 
-    sticker <- sticker + geom_pkgname(package, p_x, p_y, p_color, p_family, p_size)
+    sticker <- sticker + geom_pkgname(package, p_x, p_y, p_color, p_family, p_size, ...)
 
-    sticker <- sticker + geom_url(url, x=u_x, y = u_y, color = u_color, family = u_family, size=u_size)
+    sticker <- sticker + geom_url(url, x=u_x, y = u_y, color = u_color, family = u_family, size=u_size, angle=u_angle)
+
+    if (white_around_sticker)
+      sticker <- sticker + white_around_hex(size = h_size)
+
+    sticker <- sticker + theme_sticker(size = h_size)
 
     save_sticker(filename, sticker, dpi = dpi)
     invisible(sticker)
 }
 
-##' empty hexagon
-##'
-##'
-##' @title hexagon
-##' @param size size of border
-##' @param fill color of hexagon
-##' @param color border color of hexagon
-##' @return hexagon
-##' @export
-##' @author guangchuang yu
-hexagon <- function(size=1.2, fill="#1881C2", color="#87B13F") {
-    ggplot() + geom_hexagon(size=size, fill=fill, color=color) + theme_sticker(size)
-}
+# ##' empty hexagon
+# ##'
+# ##'
+# ##' @title hexagon
+# ##' @param size size of border
+# ##' @param fill color of hexagon
+# ##' @param color border color of hexagon
+# ##' @return hexagon
+# ##' @export
+# ##' @author guangchuang yu
+# hexagon <- function(size=1.2, fill="#1881C2", color="#87B13F") {
+#     ggplot() + geom_hexagon(size=size, fill=fill, color=color) + theme_sticker(size)
+# }
+
 
 ##' @importFrom grDevices rgb
 ##' @author Johannes Rainer with modification from Guangchuang Yu and Sebastian Gibb
@@ -130,6 +147,12 @@ geom_pkgname <- function(package, x=1, y=1.4, color="#FFFFFF", family="Aller_Rg"
 ##' @importFrom sysfonts font_add
 ##' @importFrom showtext showtext_auto
 load_font <- function(family) {
+    ## load the font packed in the hexSticker package,
+    ## otherwise, load system fonts
+    ##
+    ## google font can be supported via `showtext`,
+    ## see https://github.com/GuangchuangYu/hexSticker#google-fonts
+    ## 
     if (family == "Aller") {
         family <- "Aller_Rg"
     }
@@ -140,7 +163,7 @@ load_font <- function(family) {
     if (any(i)) {
         font_add(family, fonts[which(i)[1]])
         showtext_auto()
-    }
+    } 
     return(family)
 }
 
@@ -203,6 +226,40 @@ geom_hexagon <- function(size=1.2, fill="#1881C2", color="#87B13F") {
                  size = size, fill = fill, color = color)
 }
 
+
+white_around_hex <- function(size = 1.2) {
+  # copied from theme_sticker
+  center <- 1
+  radius <- 1
+  h <- radius
+  w <- sqrt(3)/2 * radius
+  m <- 1.02
+  x_lims <- c(center-w*m , center+w*m)
+  y_lims <- c(center-h*m , center+h*m)
+  # starting at left, upper and going around counter-clockwise
+  x_vertices <- 1+c(rep(-sqrt(3)/2, 2), 0, rep(sqrt(3)/2, 2), 0)
+  y_vertices <- 1+c(0.5, -0.5, -1, -0.5, 0.5, 1)
+
+  list(
+  ggplot2::geom_polygon(mapping = aes_(x = ~x, y = ~y),
+                        data = data.frame(x = c(x_lims[1], x_lims[1], x_vertices[6]),
+                                          y = c(y_vertices[1], y_lims[2], y_lims[2])),
+                        fill = 'white'),
+  ggplot2::geom_polygon(mapping = aes_(x = ~x, y = ~y),
+                        data = data.frame(x = c(x_vertices[6], x_lims[2], x_lims[2]),
+                                          y = c(y_lims[2], y_lims[2], y_vertices[5])),
+                        fill = 'white'),
+  ggplot2::geom_polygon(mapping = aes_(x = ~x, y = ~y),
+                        data = data.frame(x = c(x_vertices[3], x_lims[2], x_lims[2]),
+                                          y = c(y_lims[1], y_vertices[4], y_lims[1])),
+                        fill = 'white'),
+  ggplot2::geom_polygon(mapping = aes_(x = ~x, y = ~y),
+                        data = data.frame(x = c(x_lims[1], x_lims[1], x_vertices[3]),
+                                          y = c(y_lims[1], y_vertices[2], y_lims[1])),
+                        fill = 'white')
+  )
+}
+
 ##' sticker theme
 ##'
 ##'
@@ -225,16 +282,17 @@ theme_sticker <- function(size=1.2, ...) {
     h <- radius
     w <- sqrt(3)/2 * radius
     m <- 1.02
-    list(theme_transparent() +
-         theme(plot.margin = margin(b = -.2, l= -.2, unit = "lines"),
-               strip.text = element_blank(),
-               line = element_blank(),
-               text = element_blank(),
-               title = element_blank(), ...),
-         coord_fixed(),
-         scale_y_continuous(expand = c(0, size/sqrt(3)/44), limits = c(center-h*m , center+h*m )),
-         scale_x_continuous(expand = c(0, 0), limits = c(center-w*m , center+w*m ))
-         )
+    list(
+      theme_transparent() +
+        theme(plot.margin = margin(b = -.2, l= -.2, unit = "lines"),
+              strip.text = element_blank(),
+              line = element_blank(),
+              text = element_blank(),
+              title = element_blank(), ...),
+      coord_fixed(),
+      scale_y_continuous(expand = c(0, 0), limits = c(center-h*m , center+h*m )),
+      scale_x_continuous(expand = c(0, 0), limits = c(center-w*m , center+w*m ))
+    )
 }
 
 
